@@ -25,17 +25,21 @@
 
 #include <QtCore/QCoreApplication>
 
+#include "linflickabletabbar.h"
 #include "linnowplayingform.h"
 #include "linvideodisplayform.h"
+#include "linhtmldisplayform.h"
 #include "lindocumentationform.h"
 #include "linaboutform.h"
 #include "linnewsfeedwidgetitem.h"
+#include "linfilterdialog.h"
 #include <QXmlStreamReader>
 #include <QFile>
 #include <QString>
 #include <QSettings>
 #include <QMaemo5InformationBox>
 #include <QFileDialog>
+#include <QAbstractKineticScroller>
 
 #include <QDebug>
 
@@ -43,20 +47,37 @@ MainWindow::MainWindow(
   QWidget *parent)
   : QMainWindow(parent),
     ui(new Ui::MainWindow),
+    flickableTabBar(0),
     nowPlayingForm(0),
-    videoDisplayForm(0)
+    videoDisplayForm(0),
+    htmlDisplayForm(0),
+    documentationForm(0),
+    aboutForm(0)
+//    filterDialog(0)
 {
   ui->setupUi(this);
 
   setAttribute(Qt::WA_Maemo5StackedWindow);
 
+  flickableTabBar = new LinFlickableTabBar(this);
+  ui->centralVerticalLayout->insertWidget(0, flickableTabBar);
+
+  connect(
+    flickableTabBar,
+    SIGNAL(itemActivated(QListWidgetItem *)),
+    this,
+    SLOT(refilter(QListWidgetItem *)));
+
   nowPlayingForm = new LinNowPlayingForm(this);
   videoDisplayForm = new LinVideoDisplayForm(this);
+  htmlDisplayForm = new LinHtmlDisplayForm(this, &qnam);
   documentationForm = new LinDocumentationForm(this);
   aboutForm = new LinAboutForm(this);
+//  filterDialog = new LinFilterDialog(this);
 
   QSettings settings("pietrzak.org", "Linguine");
 
+/*
   if (settings.contains("currentFrequencyIndex"))
   {
     ui->frequencyComboBox->setCurrentIndex(
@@ -68,6 +89,7 @@ MainWindow::MainWindow(
     ui->categoryComboBox->setCurrentIndex(
       settings.value("currentCategoryIndex").toInt());
   }
+*/
 
   retrieveNewsfeeds(settings);
 }
@@ -77,6 +99,7 @@ MainWindow::~MainWindow()
 {
   QSettings settings("pietrzak.org", "Linguine");
 
+/*
   settings.setValue(
     "currentFrequencyIndex",
     ui->frequencyComboBox->currentIndex());
@@ -84,9 +107,15 @@ MainWindow::~MainWindow()
   settings.setValue(
     "currentCategoryIndex",
     ui->categoryComboBox->currentIndex());
+*/
 
   if (nowPlayingForm) delete nowPlayingForm;
   if (videoDisplayForm) delete videoDisplayForm;
+  if (htmlDisplayForm) delete htmlDisplayForm;
+  if (documentationForm) delete documentationForm;
+  if (aboutForm) delete aboutForm;
+//  if (filterDialog) delete filterDialog;
+  if (flickableTabBar) delete flickableTabBar;
   delete ui;
 }
 
@@ -180,27 +209,30 @@ void MainWindow::retrieveNewsfeeds(
   int index = 0;
   QString name;
   QString url;
-  Frequency freq;
+  FrequencyType freq;
   ContentType category;
   MediaType media;
+  LanguageType language;
   while (index < size)
   {
     settings.setArrayIndex(index);
 
     name = settings.value("name").toString();
     url = settings.value("url").toString();
-    freq = (Frequency) settings.value("frequency").toInt();
+    freq = (FrequencyType) settings.value("frequency").toInt();
     category = (ContentType) settings.value("category").toInt();
     media = (MediaType) settings.value("media").toInt();
+    language = (LanguageType) settings.value("language").toInt();
 
     LinNewsfeedWidgetItem *nwi =
-      new LinNewsfeedWidgetItem(name, url, freq, category, media, &qnam);
+      new LinNewsfeedWidgetItem(
+        name, url, freq, category, media, language, &qnam);
 
     ui->mediaListWidget->addItem(nwi);
 
     filterItem(nwi);
 
-    nwi->parseRSS();
+//    nwi->parseRSS();
 
     ++index;
   }
@@ -258,9 +290,10 @@ void MainWindow::parseLinguineElement(
   QString refreshString;
   QString catString;
   QString mediaString;
-  Frequency freq = Other_Rate;
-  ContentType category = Other_Content;
-  MediaType media = Audio_Media;
+  FrequencyType freq = Any_Rate;
+  ContentType category = Any_Content;
+  MediaType media = Any_Media;
+  LanguageType language = Any_Language;
 
   while (!reader.atEnd())
   {
@@ -295,6 +328,10 @@ void MainWindow::parseLinguineElement(
           {
             freq = Weekly_Rate;
           }
+          else if (refreshString == "other")
+          {
+            freq = Other_Rate;
+          }
         }
 
         if (reader.attributes().hasAttribute("category"))
@@ -318,10 +355,18 @@ void MainWindow::parseLinguineElement(
 
           mediaString = reader.attributes().value("media").toString();
 
-          if (mediaString == "video")
+          if (mediaString == "audio")
+          {
+            media = Audio_Media;
+          }
+          else if (mediaString == "video")
           {
             media = Video_Media;
           }
+          else if (mediaString == "text")
+          {
+            media = Text_Media;
+          }
         }
 
         name = parseNameElement(reader);
@@ -329,20 +374,21 @@ void MainWindow::parseLinguineElement(
         settings.setArrayIndex(index++);
         LinNewsfeedWidgetItem *nwi =
           new LinNewsfeedWidgetItem(
-            name, url, freq, category, media, settings, &qnam);
+            name, url, freq, category, media, language, &qnam, settings);
 
         ui->mediaListWidget->addItem(nwi);
 
         filterItem(nwi);
 
-        nwi->parseRSS();
+//        nwi->parseRSS();
 
         // Reset all the fields:
         name.clear();
         url.clear();
-        freq = Other_Rate;
-        category = Other_Content;
-        media = Audio_Media;
+        freq = Any_Rate;
+        category = Any_Content;
+        media = Any_Media;
+        language = Any_Language;
       }
     }
   }
@@ -410,17 +456,21 @@ QString MainWindow::parseText(
 void MainWindow::filterItem(
   LinNewsfeedWidgetItem *nwi)
 {
-  // This is a hack:
-  if ( nwi->getFrequency() !=
-       (Frequency) ui->frequencyComboBox->currentIndex() )
+/*
+  if (ui->frequencyComboBox->currentIndex() > 0)
   {
-    nwi->setHidden(true);
-    return;
+    // This is a hack:
+    if ( nwi->getFrequency() !=
+       (FrequencyType) ui->frequencyComboBox->currentIndex() )
+    {
+      nwi->setHidden(true);
+      return;
+    }
   }
 
-  // More hack:
-  if (ui->categoryComboBox->currentIndex() != 2)
+  if (ui->categoryComboBox->currentIndex() > 0)
   {
+    // More hack:
     if ( nwi->getContentType() !=
          (ContentType) ui->categoryComboBox->currentIndex() )
     {
@@ -428,8 +478,17 @@ void MainWindow::filterItem(
       return;
     }
   }
+*/
 
-  nwi->setHidden(false);
+  if (flickableTabBar->matchesCurrentCategory(nwi))
+  {
+    nwi->setHidden(false);
+    nwi->parseRSS();
+  }
+  else
+  {
+    nwi->setHidden(true);
+  }
 }
   
 
@@ -441,7 +500,7 @@ void MainWindow::on_mediaListWidget_itemActivated(QListWidgetItem *item)
 
   if (!nwi) return;
 
-  MediaType media = nwi->getMediaType();
+  MediaType media = nwi->getMedia();
 
   if (media == Audio_Media)
   {
@@ -472,6 +531,19 @@ void MainWindow::on_mediaListWidget_itemActivated(QListWidgetItem *item)
 
     videoDisplayForm->show();
   }
+  else if (media == Text_Media)
+  {
+    htmlDisplayForm->displayText(
+      nwi->getSourceUrl());
+
+    htmlDisplayForm->show();
+  }
+}
+
+
+void MainWindow::on_actionManage_Categories_triggered()
+{
+  // Need to fill this in
 }
 
 
@@ -540,8 +612,21 @@ void MainWindow::on_actionAbout_triggered()
 }
 
 
-void MainWindow::refilter()
+void MainWindow::refilter(
+  QListWidgetItem *tabItem)
 {
+/*
+  QAbstractKineticScroller *scroller =
+    flickableTabBar->property("kineticScroller")
+      .value<QAbstractKineticScroller *>();
+
+  if (scroller->state() != QAbstractKineticScroller::Inactive)
+  {
+    // Don't do anything if the user is pushing the tab bar around...
+    return;
+  }
+*/
+
   int count = ui->mediaListWidget->count();
 
   int index = 0;
@@ -558,6 +643,7 @@ void MainWindow::refilter()
 }
 
 
+/*
 void MainWindow::on_frequencyComboBox_currentIndexChanged(int index)
 {
   refilter();
@@ -568,3 +654,12 @@ void MainWindow::on_categoryComboBox_currentIndexChanged(int index)
 {
   refilter();
 }
+*/
+
+
+/*
+void MainWindow::on_filterButton_clicked()
+{
+  filterDialog->exec();
+}
+*/
